/*
             LUFA Library
     Copyright (C) Dean Camera, 2021.

  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/

/*
  Copyright 2021  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaims all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *
 *  Main source file for the Minimal demo. This file contains the main tasks of the demo and
 *  is responsible for the initial application hardware configuration.
 */

#define INCLUDE_FROM_MINIMAL_C
#define CYCLES 30000

#include <util/delay.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "Minimal.h"
#include "HD44780.h"


// Declarations
#define	NB_ROWS	2
#define	NB_COLS	16

#define LED1       4
#define LED2       6
#define LED3       7
#define BOUTON1    5
#define BOUTON2    6
#define BOUTON3    6
#define BOUTON4    7

/**
 * Read data from USB
 */
void read_usb() {
    static int pos = 0;
    Endpoint_SelectEndpoint(ICHUT_OUT_EPADDR);

    if (Endpoint_IsOUTReceived()) {
        if (Endpoint_IsReadWriteAllowed()) {
            int nb = Endpoint_BytesInEndpoint();
            unsigned char cmd = Endpoint_Read_8();
            if(cmd == 0) {
                pos = 0;
            }
            Endpoint_Read_EStream_LE((void*) pos, nb-1, NULL);
            pos += nb-1;
        }
    }
    Endpoint_ClearOUT();
}

static void write_char(const int x, const int y, const char c) {
    int address=HD44780_XY2Adrr(NB_ROWS,NB_COLS,y,x);
    HD44780_WriteCommand(LCD_ADDRSET|address);
    HD44780_WriteData(c);
}

static void write_string(const int x, const int y, const char s[]) {
    for(int i=0;i<strlen(s);i++) {
        int address=HD44780_XY2Adrr(NB_ROWS,NB_COLS,y,x+i);
        HD44780_WriteCommand(LCD_ADDRSET|address);
        HD44780_WriteData(s[i]);
    }
}

static int print_word(const char word[], const int dis[]) {
    int res = 1;
    for (int i=0;i<strlen(word);i++) {
        if (dis[i]) {
            write_char(i, 0, word[i]);
        } else {
            write_char(i, 0, '_');
            res = 0;
        }
    }
    return res;
}

static int guess(char letter, char word[], int **dis[]) {
    int found = 0;
    for (int i=0;i<strlen(word);i++) {
        if (word[i] == letter) {
            dis[i] = 1;
            found = 1;
        }
    }
    return found;
}

static char * pick_word(char mots[]) {

    // Compte le nombre de mots en comptant les '0'.
    int nombreDeMots = 1; // Il y a au moins un mot.
    for (int i = 0; i < strlen(mots); i++) {
        if (mots[i] == '0') {
            nombreDeMots++;
        }
    }

    // Génère un indice aléatoire pour choisir un mot.
    int indiceAleatoire = rand() % nombreDeMots;

    // Initialise un pointeur pour parcourir les mots.
    char *motCourant = strtok(mots, "0");

    // Parcourt les mots pour trouver le mot sélectionné.
    for (int i = 0; i < indiceAleatoire; i++) {
        motCourant = strtok(NULL, "0");
    }

    return motCourant;
}

static int mod(int x, const int mod) {
    while (x+1 > mod) {
        x = x-mod;
    }
    while (x < 0) {
        x = x+mod;
    }
    return x;
}

static void up_led(const int nb) {
    if (nb < 3) {
        PORTD &= ~(1<<LED3);
    } else {
        PORTD |= (1<<LED3);
    }
    if (nb < 2) {
        PORTD &= ~(1<<LED2);
    } else {
        PORTD |= (1<<LED2);
    }
    if (nb < 1) {
        PORTD &= ~(1<<LED1);
    } else {
        PORTD |= (1<<LED1);
    }
}

/** Main program entry point. This routine configures the hardware required by the application, then
 *  enters a loop to run the application tasks in sequence.
 */
int main(void) {
    //uC configuration
    CLKSEL0 = 0b00010101;     // sélection de l'horloge externe
    CLKSEL1 = 0b00001111;     // minimum de 8Mhz
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;
    MCUCR |= (1<<JTD);
    MCUCR |= (1<<JTD);

    //Configuration et séquence d'initilisation de l'écran LCD
    HD44780_Initialize();
    HD44780_WriteCommand(LCD_ON|CURSOR_NONE);
    HD44780_WriteCommand(LCD_CLEAR);
    _delay_ms(500);

    //Bienvenue
    write_string(3, 0, "The  Pendu");
    write_string(0, 1, "par Remi et Loic");
    
    //
    SetupHardware();
    GlobalInterruptEnable();
    
    //Configuration des leds et boutons
    DDRD |= (1<<LED1); // LED 1
    DDRD |= (1<<LED2); // LED 2
    DDRD |= (1<<LED3); // LED 3
    DDRB &= ~(1<<BOUTON1); // BOUTON 1
    PORTB |= (1<<BOUTON1);
    DDRB &= ~(1<<BOUTON2); // BOUTON 2
    PORTB |= (1<<BOUTON2);
    DDRC &= ~(1<<BOUTON3); // BOUTON 3
    PORTC |= (1<<BOUTON3);
    DDRC &= ~(1<<BOUTON4); // BOUTON 4
    PORTC |= (1<<BOUTON4);

    //
    //eeprom_write_block("A0BB0", (void*)0, 4);
    //static char* EEMEM lesMots = 0;
    char lesMots[] = "GUEPE0GOUTTE0COUSIN0BARBE0BISCUIT0CUISINIERE0PIGEON0DIMANCHE0POUSSETTE";

    int r = 0;
    while (PINC & (1<<BOUTON3)) {
        r++;
    }
    srand(r);

    char *leMot = pick_word(lesMots);
    //char
    
    int letterDiscover[strlen(leMot)];
    for (int i=0;i<strlen(leMot);i++) {
        letterDiscover[i] = 0;
    }
    
    char alpha[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    int letter = 0;
    // int game = 1;
    int lives = 3;
    PORTD |= (1<<LED1);
    PORTD |= (1<<LED2);
    PORTD |= (1<<LED3);

    int i = 0;
    
    _delay_ms(500);
    HD44780_WriteCommand(LCD_CLEAR);
    _delay_ms(100);

    print_word(leMot, letterDiscover);
    write_char(1, 1, alpha[letter]);

    while (1) {
        i++;
        if (i > CYCLES) {
            if (!(PINB & (1<<BOUTON1))) {
                letter = mod(letter-1, 26);
                write_char(1, 1, alpha[letter]);
            } else if (!(PINB & (1<<BOUTON2))) {
                letter = mod(letter+1, 26);
                write_char(1, 1, alpha[letter]);
            } else if (!(PINC & (1<<BOUTON4))) {
                if (guess(alpha[letter], leMot, &letterDiscover)) {
                    if (print_word(leMot, letterDiscover)) {
                        // game = 0;
                        write_string(12, 0, "YOU");
                        write_string(12, 1, "WIN");
                    }
                } else {
                    lives--;
                    up_led(lives);
                    if (lives == 0) {
                        // game = 0;
                        write_string(12, 0, "GAME");
                        write_string(12, 1, "OVER");
                    }
                }
            }
            i = 0;
        } else {
            USB_USBTask();
            read_usb();
        }
    }//*/
}


/**
 * Configures the board hardware and chip peripherals for the demo's functionality.
 */
void SetupHardware(void) {
#if (ARCH == ARCH_AVR8)
    /* Disable watchdog if enabled by bootloader/fuses */
    MCUSR &= ~(1 << WDRF);
    wdt_disable();

    /* Disable clock division */
    clock_prescale_set(clock_div_1);
#elif (ARCH == ARCH_XMEGA)
    /* Start the PLL to multiply the 2MHz RC oscillator to 32MHz and switch the CPU core to run from it */
    XMEGACLK_StartPLL(CLOCK_SRC_INT_RC2MHZ, 2000000, F_CPU);
    XMEGACLK_SetCPUClockSource(CLOCK_SRC_PLL);

    /* Start the 32MHz internal RC oscillator and start the DFLL to increase it to 48MHz using the USB SOF as a reference */
    XMEGACLK_StartInternalOscillator(CLOCK_SRC_INT_RC32MHZ);
    XMEGACLK_StartDFLL(CLOCK_SRC_INT_RC32MHZ, DFLL_REF_INT_USBSOF, F_USB);

    PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;
#endif

    /* Hardware Initialization */
    USB_Init();
}

/** Event handler for the USB_Connect event. This indicates that the device is enumerating via the status LEDs. */
void EVENT_USB_Device_Connect(void) {
}

/** Event handler for the USB_Disconnect event. This indicates that the device is no longer connected to a host via
 *  the status LEDs.
 */
void EVENT_USB_Device_Disconnect(void) {
}

/** Event handler for the USB_ConfigurationChanged event. This is fired when the host set the current configuration
 *  of the USB device after enumeration - the device endpoints are configured.
 */
void EVENT_USB_Device_ConfigurationChanged(void) {
    Endpoint_ConfigureEndpoint(ICHUT_OUT_EPADDR, EP_TYPE_INTERRUPT, ICHUT_OUT_EPSIZE, 1);
}

/** Event handler for the USB_ControlRequest event. This is used to catch and process control requests sent to
 *  the device from the USB host before passing along unhandled control requests to the library for processing
 *  internally.
 */
void EVENT_USB_Device_ControlRequest(void) {
}
